--
-- InteractiveButtons v3.1.1
--
-- Interface: 1.4.4.0 1.4.4RC8
--
-- Specialization for an interactive control
--
-- @author  	Manuel Leithner (SFM-Modding)
-- @version 	v2.1
-- @date  		29/08/11
-- @history:	v1.0 - Initial version
--				v2.0 - converted to ls2011
--				v2.1 - improvements
-- 20/02/2017	v3.0 - converted to work with ls2017
-- 30/07/2017	v3.1 - extra features added
-- 16/08/2017	v3.1.1 - changes as seen below
--
--
-- v3.0 Converted and adapted to LS/FS 2017 including MP Support.  -  GtX | Andy
-- v3.1 Added support for lockMovingTools.lua, Cruise Control (On/Off), Cruise Control speed change, pipe folding and Harvester Chopper state toggle  -   (As Requested)  -  GtX | Andy
-- v3.1.1 Fixed issue with ignition in MP.  -  GtX | Andy
--
-- Support for v3.1 and above.  @  http://ls-modcompany.com
--
-- ADDED SINCE V2.1:               Support for ls2017 Base Features: Front, High, Workfront & WorkRear lights.
--								   Engine Start/Stop, Warning Flashers, Beacon Lights, CrabSteering State Buttons 1, 2, 3 & 4.
--							       FLT Mode and Folding Parts. Support for Cablight.lua, Support for lockMovingTools.lua, Cruise Control (On/Off)
--								   Cruise Control speed change Up/Down, Pipe State Changing and Chopper Toggle.
--
--
-- free for noncommerical-usage
--
--
--
--	NOTES: All functions are for "User Vehicle Only" Connected Trailer & Implement actions coming soon. Suggestions welcome @ http://ls-modcompany.com
--
----------------------------------------------------------------------------------------------------------------------------------------------------------------------
--		FUNCTIONS AVAILABLE				|					EVENT				|			SCRIPT | BASE GAME			|				USED FOR					--
----------------------------------------------------------------------------------------------------------------------------------------------------------------------
--		Ignition						|			toggleMotorIgnition			|				BASE GAME				|	Start and stop engine					--
--		Beacon Lights					|			toggleBeaconLights			|				BASE GAME				|	Activate Beacon Lights					--
--		Warning/Hazard Lights			|			toggleWarningLights			|				BASE GAME				|	Activate Hazard Lights					--
--		CabLight On/Off					|			toggleCablight				|			SCRIPT = Cablight.lua		|	Activate Cab Light						--
--		Front Lights					|			toggleFrontLights			|				BASE GAME				|	Activate Front Lights					--
--		High Beam Lights				|			toggleHighBeamLights		|				BASE GAME				|	Activate High Lights					--
--		Front Work Lights				|			toggleFrontWorkLights		|				BASE GAME				|	Activate Front Work Lights				--
--		Rear Work Lights				|			toggleRearWorkLights		|				BASE GAME				|	Activate Rear Work Lights				--
--		Front Lift Turn Mode			|			toggleFLT					|				BASE GAME				|	Swap between CrabSteering State 1 & 2	--
--		CrabSteering Mode 1				|			toggleCrabState1			|				BASE GAME				|	Activate ONLY of CrabSteering State 1	--
--		CrabSteering Mode 2				|			toggleCrabState2			|				BASE GAME				|	Activate ONLY of CrabSteering State 2	--
--		CrabSteering Mode 3				|			toggleCrabState3			|				BASE GAME				|	Activate ONLY of CrabSteering State 3	--
--		CrabSteering Mode 4				|			toggleCrabState4			|				BASE GAME				|	Activate ONLY of CrabSteering State 4	--
--		Fold Parts (Main)				|			toggleFoldingParts			|				BASE GAME				|	Fold or Unfold Vehicle					--
--		Fold Parts (Middle)				|			toggleFoldingMiddleParts	|				BASE GAME				|	Middle Fold (e.g part fold) Vehicle		--
--		Lock Moving Parts				|			toggleLockMovingTools		|		SCRIPT = lockMovingTools.lua	|	Lock Mouse control tools on vehicle		--
--		Cruise Control on / off			|			toggleCruiseState			|				BASE GAME				|	Turn on and off Cruise Control			--
--		Cruise Control Increase			|			toggleCruiseSpeedRaise		|				BASE GAME				|	Increase Cruise Control set speed		--
--		Cruise Control Lower			|			toggleCruiseSpeedLower		|				BASE GAME				|	Lower Cruise Control set speed			--
--		Harvester Chopper State			|			toggleHarvesterChopper		|				BASE GAME				|	Change Chopper State					--
--		Vehicle Pipe State				|			toggleVehiclePipe			|				BASE GAME				|	Change Vehicle Pipe State				--
----------------------------------------------------------------------------------------------------------------------------------------------------------------------
--		More features are being added all the time. Support and questions for this version of the Script available: http://ls-modcompany.com	GtX | Andy			--
----------------------------------------------------------------------------------------------------------------------------------------------------------------------
--
--
--
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--	HOW TO USE - "vehicle.xml" - Information:
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--	
--	name			=	This is the displayed name of the function (Must be entered in the modDesc as in example below).
--	mark			=	This is the icon/mark that is displayed when IC is active and shows the user where to click.
--	size			=	This is the click radius of the mark. (Recommended 0.01 - 0.03)
--	event			=	This is the action that will be peformed when mark is clicked in game. (Only "events" listed above are supported at this stage)
--	playClickSound	=	If "true" then this will enable a click sound when an action is peformed on a "mark".
--	onMessage		=	This is the message displayed when function is not active. (This is written in string format so " %s " must be added at the place you would like the "name" displayed) See Example.
--	offMessage		=	This is the message displayed when function is active. 	   (This is written in string format so " %s " must be added at the place you would like the "name" displayed) See Example.
--
--	NOTE!!!!		=	If no "onMessage" or "offMessage" is set then the defualt "Turn ON %s" and "Turn OFF %s" will be used. (This is drawn from Base Game so all languages are supported).
--
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- EXAMPLE FOR XML:
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--
--	<buttons>
--		<button name="icFrontLights" 	mark="0>12|0|8|1" 	size="0.02" event="toggleFrontLights" 		playClickSound="true" />
--		<button name="icIgnition"  		mark="0>12|0|8|5"	size="0.03" event="toggleMotorIgnition" />
--		<button name="icToolsLock" 		mark="0>12|0|8|8" 	size="0.03" event="toggleLockMovingTools" 	playClickSound="true" onMessage="ic_button_locked" 		offMessage="ic_button_unlocked"/>
--		<button name="icCruise" 		mark="0>12|0|8|9" 	size="0.02" event="toggleCruiseState" 		playClickSound="true" onMessage="ic_button_activate" 	offMessage="ic_button_deactivate"/>
--		<button name="icCruise" 		mark="0>12|0|8|10" 	size="0.02" event="toggleCruiseSpeedRaise" 	playClickSound="true" onMessage="ic_button_cruiseRaise" offMessage="ic_button_cruiseMax"/>
--		<button name="icCruise" 		mark="0>12|0|8|11" 	size="0.02" event="toggleCruiseSpeedLower" 	playClickSound="true" onMessage="ic_button_cruiseLower" offMessage="ic_button_cruiseMin"/>
--	</buttons>
--
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--	EXAMPLE FOR MODDESC:
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--
--	<l10n>
--
--		<text name="ic_button_activate">
--			<en>Activate %s</en>
--			<de>%s anschalten</de>		
--			<fr>activer %s</fr>
--			<es>Activar %s</es>
--			<it>Attiva %s</it>
--		</text>
--		
--		<text name="ic_button_deactivate">
--			<en>Deactivate %s</en>
--			<de>%s deaktivieren</de>		
--			<fr>désactiver %s</fr>
--			<es>Desactivar %s</es>
--			<it>Disattiva %s</it>
--		</text>
--
--		<text name="icCruise">
--			<en>Cruise Control</en>
--			<de>Tempomat</de>	
--          <fr>le régulateur de vitesse</fr>
--			<es>control de crucero</es>
--			<it>regolazione velocità</it>
--		</text>
--		
--	</l10n>		
--
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--	END:
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--


InteractiveButtons = {};

function InteractiveButtons.prerequisitesPresent(specializations)
	if not SpecializationUtil.hasSpecialization(InteractiveControl, specializations) then print("Warning: Specialization InteractiveButtons v3.1.1 needs the specialization InteractiveControl v4.1"); end;
    return SpecializationUtil.hasSpecialization(InteractiveControl, specializations);
end;

function InteractiveButtons:load(savegame)

	local i=0;
	while true do
		local buttonName = string.format("vehicle.interactiveComponents.buttons.button(%d)", i);	
		if not hasXMLProperty(self.xmlFile, buttonName) then
			break;
		end;
		local name = Utils.getNoNil(g_i18n:getText(getXMLString(self.xmlFile, buttonName .. "#name")), "ERROR");
		local mark = Utils.indexToObject(self.components, getXMLString(self.xmlFile, buttonName .. "#mark"));
		local highlight = getChildAt(mark, 0);
		local size = Utils.getNoNil(getXMLFloat(self.xmlFile, buttonName .. "#size"), 0.1);
		local event = getXMLString(self.xmlFile, buttonName .. "#event");
		local onMessage = g_i18n:getText(Utils.getNoNil(getXMLString(self.xmlFile, buttonName .. "#onMessage"), "action_turnOnOBJECT"));
		local offMessage =  g_i18n:getText(Utils.getNoNil(getXMLString(self.xmlFile, buttonName .. "#offMessage") , "action_turnOffOBJECT"));
		
		local playBcSound = Utils.getNoNil(getXMLBool(self.xmlFile, buttonName .. "#playClickSound"), false);
	
		local button = Button:new(nil, highlight, name, mark, size, event, self, onMessage, offMessage, playBcSound);
		
		button.synch = Utils.getNoNil(getXMLBool(self.xmlFile, buttonName .. "#synch"), true);

		table.insert(self.interactiveObjects, button);
		i = i + 1;
	end;
	
end;

function InteractiveButtons:delete()
end;

function InteractiveButtons:mouseEvent(posX, posY, isDown, isUp, button)
end;

function InteractiveButtons:keyEvent(unicode, sym, modifier, isDown)
end;

function InteractiveButtons:update(dt)	
end;

function InteractiveButtons:draw()
end;



--
-- Button Class
-- Specifies an interactive Button
--
-- SFM-Modding
-- @author  Manuel Leithner
-- @date  29/08/11
--
--
-- 2017 v3.0 Feature Added 20/02/2017 by GtX. All credit and ownership to Manuel Leithner. 

Button = {};

function Button:new(node, highlight, name, mark, size, event, vehicle, onMessage, offMessage, playBcSound)

	local Button_mt = Class(Button, InteractiveComponentInterface);	
    local instance = InteractiveComponentInterface:new(node, highlight, name, mark, size, onMessage, offMessage, Button_mt);

	instance.vehicle = vehicle;
	instance.event = event;
	instance.playBcSound = playBcSound
	
	return instance;	
end;

function Button:delete()
	InteractiveComponentInterface.delete(self);
end;

function Button:mouseEvent(posX, posY, isDown, isUp, button)
	InteractiveComponentInterface.mouseEvent(self, posX, posY, isDown, isUp, button);
end;

function Button:keyEvent(unicode, sym, modifier, isDown)
	InteractiveComponentInterface.keyEvent(self, unicode, sym, modifier, isDown);
end;

function Button:update(dt)
	if self.vehicle ~= nil then
		if self.event == "toggleMotorIgnition" then		
			self.isOpen = self.vehicle.isMotorStarted;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleBeaconLights" then
			self.isOpen = self.vehicle.beaconLightsActive;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleWarningLights" then
			self.isOpen = self.vehicle.turnLightState == Lights.TURNLIGHT_HAZARD;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCablight" then
			self.isOpen = self.vehicle.cl.turnOn;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontLights" then
			if self.vehicle.numLightTypes >= 1 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^0) == 1 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleHighBeamLights" then
			if self.vehicle.numLightTypes >= 4 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^3) == 8 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontWorkLights" then
			if self.vehicle.numLightTypes >= 3 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^2) == 4 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleRearWorkLights" then
			if self.vehicle.numLightTypes >= 2 then
				self.isOpen = (bitAND(self.vehicle.lightsTypesMask, 2^1) == 2 );
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFLT" then
			self.isOpen = self.vehicle.crabSteering.state == 1;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState1" then
			self.isOpen = self.vehicle.crabSteering.state == 1;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState2" then
			self.isOpen = self.vehicle.crabSteering.state == 2;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState3" then
			self.isOpen = self.vehicle.crabSteering.state == 3;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState4" then
			self.isOpen = self.vehicle.crabSteering.state == 4;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingParts" then
			self.isOpen = self.vehicle:getToggledFoldDirection() > 0;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingMiddleParts" then
			self.isOpen = self.vehicle:getToggledFoldMiddleDirection() > 0;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleLockMovingTools" then
			self.isOpen = self.vehicle.lmt.toolState == 2;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCruiseState" then
			self.isOpen = self.vehicle.cruiseControl.state == Drivable.CRUISECONTROL_STATE_ACTIVE;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCruiseSpeedRaise" then
			local single = false
			self.isOpen = self.vehicle.cruiseControl.speed == self.vehicle.cruiseControl.maxSpeed
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCruiseSpeedLower" then
			self.isOpen = self.vehicle.cruiseControl.speed == self.vehicle.cruiseControl.minSpeed
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleHarvesterChopper" then
			self.isOpen = self.vehicle.isStrawEnabled
		end;
	end;	
	if self.vehicle ~= nil then
		if self.event == "toggleVehiclePipe" then
			self.isOpen = self.vehicle.pipeTargetState == 2
		end;
	end;

	InteractiveComponentInterface.update(self, dt);
end;

function Button:draw()
	InteractiveComponentInterface.draw(self);
end;

function Button:doAction(noEventSend, forceAction)
	InteractiveComponentInterface.doAction(self, forceAction);
	
	if self.vehicle ~= nil then
		if self.playBcSound ~= nil and self.vehicle:getIsActiveForSound() then
			if self.playBcSound then
				SoundUtil.playSample(g_currentMission.sampleToggleLights, 1, 0, 1);
			end;
		end;
	end;
	
	
	if self.vehicle ~= nil then		
		if self.event == "toggleMotorIgnition" then
			if forceAction == nil then
				if not g_currentMission.missionInfo.automaticMotorStartEnabled then
					if self.vehicle.isMotorStarted then
						self.vehicle:stopMotor();
					else
						self.vehicle:startMotor();
					end;
				end;
			end;
		end;	
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleBeaconLights" then
			if forceAction == nil then
				self.vehicle:setBeaconLightsVisibility(not self.vehicle.beaconLightsActive);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleWarningLights" then
			if forceAction == nil then
			local state = Lights.TURNLIGHT_OFF;
                if self.vehicle.turnLightState ~= Lights.TURNLIGHT_HAZARD then
                    state = Lights.TURNLIGHT_HAZARD;
                end
                self.vehicle:setTurnLightState(state);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCablight" then
			if forceAction == nil then
				local state = not self.vehicle.cl.turnOn;
				self.vehicle:setCablight(state, true);
			end;
		end;	
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontLights" then
			if self.vehicle.numLightTypes >= 1 then
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^0) == 1 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^0));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleHighBeamLights" then
			if self.vehicle.numLightTypes >= 4 then
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^3) == 8 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^3));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFrontWorkLights" then
			if self.vehicle.numLightTypes >= 3 then
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^2) == 4 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^2));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleRearWorkLights" then
			if self.vehicle.numLightTypes >= 2 then
				if forceAction ~= (bitAND(self.vehicle.lightsTypesMask, 2^1) == 2 ) then
					self.vehicle:setLightsTypesMask(bitXOR(self.vehicle.lightsTypesMask, 2^1));
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFLT" then
			if forceAction == nil then
				if self.vehicle.crabSteering.state == 1 then
					self.vehicle:setCrabSteering(2);
				else
					self.vehicle:setCrabSteering(1)
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState1" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(1);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState2" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(2);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState3" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(3);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCrabState4" then
			if forceAction == nil then
				self.vehicle:setCrabSteering(4);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingParts" then
			if forceAction == nil then
				if self.vehicle:getIsFoldAllowed() then
	                if self.vehicle:getToggledFoldDirection() == self.turnOnFoldDirection then
	                    self.vehicle:setFoldState(self.vehicle:getToggledFoldDirection(), true);
	                else
	                    self.vehicle:setFoldState(self.vehicle:getToggledFoldDirection(), false);
	                end
	            end
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleFoldingMiddleParts" then
			if forceAction == nil then
				local direction = self.vehicle:getToggledFoldMiddleDirection();
				if direction ~= 0 then
					if direction == self.turnOnFoldDirection then
						self.vehicle:setFoldState(direction, false)
					else
						self.vehicle:setFoldState(direction, true)
					end
				end
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleLockMovingTools" then
			if forceAction == nil then
				if self.vehicle.lmt.toolState == 1 then
					self.vehicle.lmt.toolState = 2
				else
					self.vehicle.lmt.toolState = 1
				end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCruiseState" then
			if forceAction == nil then
				if self.vehicle.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
                    self.vehicle:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE);
                else
                    self.vehicle:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);
                end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCruiseSpeedRaise" then
			if forceAction == nil then
				self.vehicle:setCruiseControlMaxSpeed(self.vehicle.cruiseControl.speed + 1);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleCruiseSpeedLower" then
			if forceAction == nil then
				self.vehicle:setCruiseControlMaxSpeed(self.vehicle.cruiseControl.speed - 1);
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleHarvesterChopper" then
			if forceAction == nil then
				if self.vehicle.isStrawEnabled then
                    self.vehicle:setIsStrawEnabled(false)
                else
                    self.vehicle:setIsStrawEnabled(true)
                end;
			end;
		end;
	end;
	if self.vehicle ~= nil then
		if self.event == "toggleVehiclePipe" then
			if forceAction == nil then
				local nextState = self.vehicle.pipeTargetState+1;
				if nextState > self.vehicle.pipeNumStates then
					nextState = 1;
				end;
				if self.vehicle:getIsPipeStateChangeAllowed(nextState) then
                    self.vehicle:setPipeState(nextState);
                elseif nextState ~= 1 and self:getIsPipeStateChangeAllowed(1) then
                    self.vehicle:setPipeState(1);
                end
			end;
		end;
	end;
end;

function Button:onEnter(dt)
	InteractiveComponentInterface.onEnter(self, dt);
end;

function Button:onExit(dt)
	InteractiveComponentInterface.onExit(self, dt);
end;

function Button:setActive()
	InteractiveComponentInterface.setActive(self, isActive);
end;

function Button:setVisible(isVisible)
	InteractiveComponentInterface.setVisible(self, isVisible);
end;